@@ -1,8 +1,10 @@
 from .javascript import JavaScript, JS
 from .empty import Empty
+from .sid import SocketID
 
 __all__ = [
     "JavaScript",
     "JS",
     "Empty",
+    "SocketID",
 ]
